{
  -----------------------------------------------------------------------------

  GPEDIT.H - Definitions and prototypes for Group Policy

  Copyright 1997-1999, Microsoft Corporation

 ------------------------------------------------------------------------------

 Translation for Delphi by Deefaze :: 08/20/2009

 ------------------------------------------------------------------------------
}
unit GPEdit;

interface

{$X+} {$ALIGN ON} {$MINENUMSIZE 4}

uses
  Windows, SysUtils, ActiveX;

type
  HPROPSHEETPAGE  = Pointer;
  PHPROPSHEETPAGE = ^HPROPSHEETPAGE;

  LPOLESTR = POleStr;


{ Terminology

    Group Policy Editor  -  The tool to view a Group Policy Object.

    Group Policy Object  -  A collection of administrator defined policies.
                           Each Group Policy Object (GPO) has both file system
                           and Active Directory storage available to it.

    IGPEInformation      -  The interface MMC Snapin Extensions use to
                           talk to the Group Policy Editor.

    IGroupPolicyObject   -  The interface used to create/edit a GPO directly
                           without going through the Group Policy Editor
}



{ Group Policy Editor MMC SnapIn GUID
}
const
  CLSID_GPESnapIn              : TGUID = '{8FC0B734-A0E1-11d1-A7D3-0000F87571E3}';



{ Group Policy Editor node ids
}
const
  // Computer Configuration\Windows Settings
  NODEID_Machine               : TGUID = '{8FC0B737-A0E1-11d1-A7D3-0000F87571E3}';

  // Computer Configuration\Software Settings
  NODEID_MachineSWSettings     : TGUID = '{8FC0B73A-A0E1-11d1-A7D3-0000F87571E3}';

  // User Configuration\Windows Settings
  NODEID_User                  : TGUID = '{8FC0B738-A0E1-11d1-A7D3-0000F87571E3}';

  // User Configuration\Software Settings
  NODEID_UserSWSettings        : TGUID = '{8FC0B73C-A0E1-11d1-A7D3-0000F87571E3}';

  // IGPEInformation interface id
  IID_IGPEInformation          : TGUID = '{8FC0B735-A0E1-11d1-A7D3-0000F87571E3}';

  // Group Policy Object class id
  CLSID_GroupPolicyObject      : TGUID = '{EA502722-A23D-11d1-A7D3-0000F87571E3}';

  // Group Policy Object interface id
  IID_IGroupPolicyObject       : TGUID = '{EA502723-A23D-11d1-A7D3-0000F87571E3}';

  // GUID that identifies the registry extension
  REGISTRY_EXTENSION_GUID      : TGUID = '{35378EAC-683F-11D2-A89A-00C04FBBCFA2}';



{ Resultant Set of Policy node ids
}
const
  // Resultant Set of Policy MMC SnapIn GUID
  CLSID_RSOPSnapIn             : TGUID = '{6DC3804B-7212-458D-ADB0-9A07E2AE1FA2}';

  // Computer Configuration\Windows Settings
  NODEID_RSOPMachine           : TGUID = '{BD4C1A2E-0B7A-4A62-A6B0-C0577539C97E}';

  // Computer Configuration\Software Settings
  NODEID_RSOPMachineSWSettings : TGUID = '{6A76273E-EB8E-45DB-94C5-25663A5f2C1A}';

  // User Configuration\Windows Settings
  NODEID_RSOPUser              : TGUID = '{AB87364F-0CEC-4CD8-9BF8-898F34628FB8}';

  // User Configuration\Software Settings
  NODEID_RSOPUserSWSettings    : TGUID = '{E52C5CE3-FD27-4402-84DE-D9A5F2858910}';

  // IRSOPInformation interface id
  IID_IRSOPInformation         : TGUID = '{9A5A81B5-D9C7-49EF-9D11-DDF50968C48D}';



{ Group Policy Object Section flags }
const
  GPO_SECTION_ROOT    = 0;  // Root
  GPO_SECTION_USER    = 1;  // User
  GPO_SECTION_MACHINE = 2;  // Machine



{ Group Policy Object types }
type
  _GROUP_POLICY_OBJECT_TYPE = (
    GPOTypeLocal = 0,                     // GPO on the local machine
    GPOTypeRemote,                        // GPO on a remote machine
    GPOTypeDS                             // GPO in the Active Directory
  );
  GROUP_POLICY_OBJECT_TYPE  = _GROUP_POLICY_OBJECT_TYPE;
  PGROUP_POLICY_OBJECT_TYPE = ^GROUP_POLICY_OBJECT_TYPE;

  TGroupPolicyObjectType = GROUP_POLICY_OBJECT_TYPE;
  PGroupPolicyObjectType = PGROUP_POLICY_OBJECT_TYPE;



{ Group Policy Hint types
}
type
  _GROUP_POLICY_HINT_TYPE = (
    GPHintUnknown = 0,                      // No link information available
    GPHintMachine,                          // GPO linked to a machine (local or remote)
    GPHintSite,                             // GPO linked to a site
    GPHintDomain,                           // GPO linked to a domain
    GPHintOrganizationalUnit                // GPO linked to a organizational unit
  );

  GROUP_POLICY_HINT_TYPE  = _GROUP_POLICY_HINT_TYPE;
  PGROUP_POLICY_HINT_TYPE = ^GROUP_POLICY_HINT_TYPE;

  TGroupPolicyHintType = GROUP_POLICY_HINT_TYPE;
  PGroupPolicyHintType = PGROUP_POLICY_HINT_TYPE;


{ IGPEInformation }
type
  IGPEInformation = interface(IUnknown)
    ['{8FC0B735-A0E1-11d1-A7D3-0000F87571E3}']

    { GetName
        Returns the unique Group Policy Object name (a GUID)

      parameters
        pszName      : contains the name on return
        cchMaxLength : is the max number of characters that can be stored in pszName
    }
    function GetName(pszName: LPOLESTR; cchMaxLength: Integer): HRESULT; stdcall;

    { GetDisplayName
        Returns the friendly display name for this Group Policy Object

      parameters
        pszName      : contains the name on return
        cchMaxLength : is the max number of characters that can be stored in pszName
    }
    function GetDisplayName(pszName: LPOLESTR; cchMaxLength: Integer): HRESULT; stdcall;

    { GetRegistryKey
        Returns a registry key handle for the requested section.  The returned
        key is the root of the registry, not the Policies subkey.  To set / read
        a value in the Policies subkey, you will need to call RegOpenKeyEx to
        open Software\Policies subkey first.

        The handle has been opened with ALL ACCESS rights.
        Call RegCloseKey on the handle when finished.

      parameters
        dwSection : is either GPO_SECTION_USER or GPO_SECTION_MACHINE
        hKey      : contains the registry key on return
    }
    function GetRegistryKey(dwSection: DWORD; var hKey: HKEY): HRESULT; stdcall;

    { GetDSPath
        Returns the Active Directory path to the root of the request section.
        The path is in ADSI name format.

      parameters
        dwSection  : is one of the GPO_SECTION_* flags
        pszPath    : contains the path on return
        cchMaxPath : is the max number of characters that can be stored in pszPath
    }
    function GetDSPath(dwSection: DWORD; pszPath: LPOLESTR; cchMaxPath: Integer): HRESULT; stdcall;

    { GtFileSysPath
        Returns the UNC path to the root of the requested section.

      parameters
        dwSection  : is one of the GPO_SECTION_* flags
        pszPath    : contains the path on return
        cchMaxPath : is the number of characters that can be stored in pszPath.
    }
    function GetFileSysPath(dwSection: DWORD; pszPath: LPOLESTR; cchMaxPath: Integer): HRESULT; stdcall;

    { GetOptions
        Returns the user preferences (options)
        Currently, there are no options defined.  This is reserved for future use.

      parameters
        dwOptions : receives a bitmask value
    }
    function GetOptions(var dwOptions: DWORD): HRESULT; stdcall;

    { GetType
        Returns the type of GPO being edited.

        The three types are:  a GPO in the Active Directory, the GPO on the local machine,
        and the GPO on a remote machine.

        Machine GPOs only have file system storage (no Active Directory storage available).
        If GetDSPath is called for a machine GPO, the function will succeed
        and the returned buffer will be the empty string ""

        Active Directory GPOs have both file system and Active Directory storage available to them.

      parameters
        gpoType : receives one of the type flags listed above.
    }
    function GetType(var gpoType: GROUP_POLICY_OBJECT_TYPE): HRESULT; stdcall;

    { GetHint
        Returns the type of Active Directory object (or machine) that could be linked to
        this GPO

        This is a hint api only.  The GPE does not know which Active Directory objects are
        linked to a particular GPO, but it can offer a hint based upon how the
        user started the GPE.

        Use this method with great caution.  Some extensions might want to
        customize their user interface based upon the scoping for this GPO,
        but it is easy to offer the wrong namespace.  Best advice is to
        always offer your full user interface, but if you choose to use this
        method, always offer your full user interface if you recieve the
        unknown hint back.

      parameters
        gpHint : receives one of the hint flags listed above.
    }
    function GetHint(var gpHint: GROUP_POLICY_HINT_TYPE): HRESULT; stdcall;

    { PolicyChange
        Informs the Group Policy Editor that policy settings have changed.
        Extensions MUST call this methold every time a change is made
        to a Group Policy Object.

      parameters
        bMachine       : specifies if machine or user policy has changed.
        bAdd           : specifies whether this is an add or delete.
        pGuidExtension : is the guid or unique name of extension that will process this GPO.
        pGuidSnapin    : is the guid or unique name of snapin that is making this call
    }
    function PolicyChanged(bMachine, bAdd: BOOL; const pGuidExtension, pGuidSnapin: TGUID): HRESULT; stdcall;

  end;
  LPGPEINFORMATION = ^IGPEInformation;



{ Group Policy Object open / creation flags
}
const
  GPO_OPEN_LOAD_REGISTRY = $00000001;  // Load the registry files
  GPO_OPEN_READ_ONLY     = $00000002;  // Open the GPO as read only



{ Group Policy Object option flags
}
const
  GPO_OPTION_DISABLE_USER    = $00000001;  // The user portion of this GPO is disabled
  GPO_OPTION_DISABLE_MACHINE = $00000002;  // The machine portion of this GPO is disabled



{ IGroupPolicyObject }
type
  IGroupPolicyObject = interface (IUnknown)
   ['{EA502723-A23D-11d1-A7D3-0000F87571E3}']
    { New
        Creates a new GPO in the Active Directory with the given friendly name
        and opens it via OpenDSGPO().  If pszDomainName contains a domain
        controller name, the GPO will be created on that DC.  If it does not
        specify a domain controller name, the method will select a DC on
        the callers behalf.

      parameters
        pszDomainName  : contains the ADSI path of the domain root
        pszDisplayName : contains the friendly display name
        dwFlags        : is a bitmask of GPO open / creation flags listed above
    }
    function New(pszDomainName, pszDisplayName: LPOLESTR; dwFlags: DWORD): HRESULT; stdcall;

    { OpenDSGPO
        Opens the specified Group Policy Object in the Active Directory
        based upon the passed in flags.  If pszPath contains a domain
        controller name, the GPO will be opened on that DC.  If it does
        not contain a domain controller name, the method will select a
        DC on the callers behalf.  If the registry is not loaded,
        GetRegistryKey() will return E_FAIL.

      parameters
        pszPath : contains the ADSI path to the GPO to open
        dwFlags : is a bitmask of GPO open / creation flags listed above
    }
    function OpenDSGPO(pszPath: LPOLESTR; dwFlags: DWORD): HRESULT; stdcall;

    { OpenLocalMachineGPO
        Opens the default Group Policy Object on this machine with the
        dwFlags options listed above.  If the registry is not loaded,
        GetRegistryKey() will return E_FAIL.

      parameters
        dwFlags : is a bitmask of GPO open / creation flags listed above
    }
    function OpenLocalMachineGPO(dwFlags: DWORD): HRESULT; stdcall;

    { OpenRemoteMachineGPO
        Opens the default Group Policy Object on a remote machine with the
        dwFlags options listed above.  If the registry is not loaded,
        GetRegistryKey() will return E_FAIL.

      parameters
        pszComputerName : contains the machine name in \\machine format
        dwFlags         : is a bitmask of GPO open / creation flags listed above
    }
    function OpenRemoteMachineGPO(pszComputerName: LPOLESTR; dwFlags: DWORD): HRESULT; stdcall;

    { Save
        Flushes the registry settings to disk and updates the revision
        number of the GPO.

      parameters
        bMachine       : specifies if machine or user should be saved.
        bAdd           : specifies whether this is an add or delete.
        pGuidExtension : is the guid or unique name of extension that will process this GPO.
        pGuid          : is a guid
    }
    function Save(bMachine, bAdd: BOOL; const pGuidExtension, pGuid: TGUID): HRESULT; stdcall;

    { Delete
        Deletes this Group Policy Object.

        After calling this method, no other methods are valid to call
        since the data will have been deleted.
    }
    function Delete: HRESULT; stdcall;

    { GetName
        Returns the unique Group Policy Object name

        For Active Directory policy objects, this is a GUID
        For the local policy object, it is the string "Local"
        For remote policy objects, it is the computername

      parameters
        pszName      : contains the name on return
        cchMaxLength : is the max number of characters that can be stored in pszName
    }
    function GetName(pszName: LPOLESTR; cchMaxLength: Integer): HRESULT; stdcall;

    { GetDisplayName
        Returns the friendly display name for this Group Policy Object

      parameters
        pszName      : contains the name on return
        cchMaxLength : is the max number of characters that can be stored in pszName
    }
    function GetDisplayName(pszName: LPOLESTR; cchMaxLength: Integer): HRESULT; stdcall;

    { SetDisplayName
        Sets the friendly display name for this Group Policy Object

      parameters
        pszName : is the new display name
    }
    function SetDisplayName(pszName: LPOLESTR): HRESULT; stdcall;

    { GetPath
        Returns the path to the Group Policy Object

        If the GPO is an Active Directory object, the path is in ADSI name format.
        If the GPO is a machine object, it is a file system path

      parameters
        pszPath    : contains the path on return
        cchMaxPath : is the max number of characters that can be stored in pszPath
    }
    function GetPath(pszPath: LPOLESTR; cchMaxPath: Integer): HRESULT; stdcall;

    { GetDSPath
        Returns the Active Directory path to the root of the request section.
        The path is in DN name format.

      parameters
        dwSection  : is one of the GPO_SECTION_* flags
        pszPath    : contains the path on return
        cchMaxPath : is the max number of characters that can be stored in pszPath
    }
    function GetDSPath(dwSection: DWORD; pszPath: LPOLESTR; cchMaxPath: Integer): HRESULT; stdcall;

    { GtFileSysPath
        Returns the UNC path to the root of the requested section.

      parameters
        dwSection  : is one of the GPO_SECTION_* flags
        pszPath    : contains the path on return
        cchMaxPath : is the number of characters that can be stored in pszPath.
    }
    function GetFileSysPath(dwSection: DWORD; pszPath: LPOLESTR; cchMaxPath: Integer): HRESULT; stdcall;

    { GetRegistryKey
        Returns a registry key handle for the requested section.  The returned
        key is the root of the registry, not the Policies subkey.  To set / read
        a value in the Policies subkey, you will need to call RegOpenKeyEx to
        open Software\Policies subkey first.

        The handle has been opened with ALL ACCESS rights.  Call RegCloseKey
        on the handle when finished.

        If the GPO was loaded / created without the registry being loaded
        this method will return E_FAIL.

      parameters
        dwSection : is either GPO_SECTION_USER or GPO_SECTION_MACHINE
        hKey      : contains the registry key on return
    }
    function GetRegistryKey(dwSection: DWORD; var hKey: HKEY): HRESULT; stdcall;

    { GetOptions
        Returns any options for this Group Policy Object

      parameters
        dwOptions : receives the GPO_OPTION_* flags
    }
    function GetOptions(var dwOptions: DWORD): HRESULT; stdcall;

    { SetOptions
        Sets any options for this Group Policy Object

        This method sets any options for this GPO.  To change
        an option, that flag must be set in the mask field.
        If the flag is in the mask field, then the dwOptions
        field is read for the current state.

        For example:  to disable the GPO, make this call

          SetOptions (GPO_OPTION_DISABLED, GPO_OPTION_DISABLED);

      parameters
        dwOptions : specifies one or more GPO_OPTION_* flags
        dwMask    : specificies which of the dwOptions to change
    }
    function SetOptions(dwOptions, dwMask: DWORD): HRESULT; stdcall;

    { GetType
        Returns the type of GPO being edited.

        The three types are:  a GPO in the Active Directory, the GPO on the local machine,
        and the GPO on a remote machine.

        Machine GPOs only have file system storage (no Active Directory storage available).
        If GetDSPath is called for a machine GPO, the function will succeed
        and the returned buffer will be the empty string ""

        Active Directory GPOs have both file system and Active Directory storage available to them.

      parameters
        gpoType : receives one of the type flags
    }
    function GetType(var gpoType: GROUP_POLICY_OBJECT_TYPE): HRESULT; stdcall;

    { GetMachineName
        Returns the machine name of the remote GPO

        This method returns the name passed to OpenRemoteMachineGPO.

      parameters
        pszName      : contains the name on return
        cchMaxLength : is the max number of characters that can be stored in pszName
    }
    function GetMachineName(pszName: LPOLESTR; cchMaxLength: Integer): HRESULT; stdcall;

    { GetPropertySheetPages
        Returns an array of property sheet pages and the number of pages
        in the array

        Note, this method will allocate memory for the array with
        LocalAlloc.  When finished, the caller should free the array
        with LocalFree

      parameters
        hPages     : address of the pointer for the array of property sheet pages
        uPageCount : receives the number of pages in the array
    }
    function GetPropertySheetPages(var hPages: PHPROPSHEETPAGE; var uPageCount: UINT): HRESULT; stdcall;
  end;
  LPGROUPPOLICYOBJECT = ^IGroupPolicyObject;



// RSOP flags
const
  RSOP_INFO_FLAG_DIAGNOSTIC_MODE = $00000001; // Running in diagnostic mode vs planning mode



type
  IRSOPInformation = interface (IUnknown)
    ['{9A5A81B5-D9C7-49EF-9D11-DDF50968C48D}']

    { GetNamespace
        Returns the namespace for the RSOP data

      parameters
        dwSection    : is either GPO_SECTION_USER or GPO_SECTION_MACHINE
        pszName      : contains the namespace on return
        cchMaxLength : is the max number of characters that can be stored in pszName
    }
    function GetNamespace(dwSection: DWORD; pszName: LPOLESTR; cchMaxLength: Integer): HRESULT; stdcall;

    { GetFlags
        Returns information about the RSOP session

      parameters
        pdwFlags : points to a DWORD which contains the flags on return
    }
    function GetFlags(var pdwFlags: DWORD): HRESULT; stdcall;

    { GetEventLogEntryText
        Returns the event log text for a specific entry

      parameters
        lpEventSource  : event log source name
        lpEventLogName : event log name
        lpEventTime    : event log time in WMI datetime format
        dwEventID      : event ID
        lpText         : Receives a pointer to a buffer containing the text.
                         The caller should free this buffer with CoTaskMemFree.
    }
    function GetEventLogEntryText(pszEventSource, pszEventLogName, pszEventTime: LPOLESTR;
      dwEventID: DWORD; out ppszText: LPOLESTR): HRESULT; stdcall;
  end;
  LPRSOPINFORMATION = IRSOPInformation;




{ CreateGPOLink
  Creates a link to a GPO for the specified Site, Domain, or Organizational Unit

  parameters
    lpGPO         : ADSI path to the GPO
    lpContainer   : ADSI path to the Site, Domain, or Organizational Unit
    fHighPriority : Create the link as the highest or lowest priority

  returns
    S_OK if successful.
}
function CreateGPOLink(lpGPO, lpContainer: LPOLESTR; fHighPriority: BOOL): HRESULT; stdcall;


{ DeleteGPOLink
  Deletes a link to a GPO for the specified Site, Domain, or Organizational Unit

  parameters
    lpGPO       : ADSI path to the GPO
    lpContainer : ADSI path to the Site, Domain, or Organizational Unit

  returns
    S_OK if successful.
}
function DeleteGPOLink(lpGPO, lpContainer: LPOLESTR): HRESULT; stdcall;


{ DeleteAllGPOLinks
  Deletes all GPO links for the specified Site, Domain, or Organizational Unit

  parameters
    lpContainer : ADSI path to the Site, Domain, or Organizational Unit

  returns
    S_OK if successful.
}
function DeleteAllGPOLinks(lpContainer: LPOLESTR): HRESULT; stdcall;



{ GPO Browser dialog
}

// Flags passed in the dwFlags field of the GPOBROWSEINFO structure
const
  GPO_BROWSE_DISABLENEW  = $00000001;   // Disables the New GPO functionality on all pages except "All"
  GPO_BROWSE_NOCOMPUTERS = $00000002;   // Removes the Computers tab
  GPO_BROWSE_NODSGPOS    = $00000004;   // Removes the Domain/OU and Sites tabs
  GPO_BROWSE_OPENBUTTON  = $00000008;   // Change the Ok button to say Open
  GPO_BROWSE_INITTOALL   = $00000010;   // Initialize the dialog focused on the All pane


type
  tag_GPOBROWSEINFO = record
    dwSize       : DWORD;                     // [i]   Initialized to the size of this structure
    dwFlags      : DWORD;                     // [i]   Flags defined above
    hwndOwner    : HWND;                      // [i]   Parent window handle (can be NULL)
    lpTitle      : LPOLESTR;                  // [i]   Title bar text. If NULL, "Browse for a Group Policy Object"
                                              //       will be the default text
    lpInitialOU  : LPOLESTR;                  // [i]   Initial Domain/Organizational Unit to open focus on
    lpDSPath     : LPOLESTR;                  // [i/o] Pointer to the buffer that receives the Active Directory GPO path
    dwDSPathSize : DWORD;                     // [i]   Size in characters of buffer given in lpDSPath
    lpName       : LPOLESTR;                  // [i/o] Pointer to a buffer that receives either the computer name or
                                              //       the friendly name of the GPO (can be NULL)
    dwNameSize   : DWORD;                     // [i]   Size in characters of buffer given in lpName
    gpoType      : GROUP_POLICY_OBJECT_TYPE;  // [o]   Specifies the type of GPO
    gpoHint      : GROUP_POLICY_HINT_TYPE;    // [o]   Specifies a hint of the GPO association
  end;

  GPOBROWSEINFO   = tag_GPOBROWSEINFO;
  LPGPOBROWSEINFO = ^GPOBROWSEINFO;

  TGpoBrowseInfo = GPOBROWSEINFO;
  PGpoBrowseInfo = LPGPOBROWSEINFO;

  const
    SizeOfGpoBrowseInfo = SizeOf(tag_GPOBROWSEINFO);
    GpoBrowseInfoDefault : TGpoBrowseInfo = (
      dwSize       : SizeOfGpoBrowseInfo;
      dwFlags      : 0;
      hwndOwner    : 0;
      lpTitle      : nil;
      lpDSPath     : nil;
      dwDSPathSize : 0;
      lpName       : nil;
      dwNameSize   : 0;
    );


{ BrowseForGPO
  Displays the GPO browser dialog

  parameters
    lpBrowseInfo : Address of a GPOBROWSEINFO structure

  returns
    S_OK if successful.
}
function BrowseForGPO(var lpBrowseInfo: GPOBROWSEINFO): HRESULT; stdcall;



{ ImportRSoPData
  Imports a data file generated by ExportRSoPData

  parameters
    lpNameSpace : Namespace to place the data in
    lpFileName  : Filename containing the data

  returns
    S_OK if successful.

  notes
    The namespace specified in lpNameSpace must exist prior to calling this function.
}
function ImportRSoPData(lpNameSpace, lpFileName: LPOLESTR): HRESULT; stdcall;


{ ExportRSoPData
  Exports a WBEM namespace containing RSoP information to a data file.
  This data file can be imported to a WBEM namespace using ImportRSoPData.

  parameters
    lpNameSpace : Namespace to read the data from
    lpFileName  : Filename to receive the data

  returns
    S_OK if successful.

  notes
    This function should be called twice.  Once for the user data and
    once for the computer data.
}
function ExportRSoPData(lpNameSpace, lpFileName: LPOLESTR): HRESULT; stdcall;



implementation

const
  GPEditLib = 'gpedit.dll';

function CreateGPOLink;     external GPEditLib name 'CreateGPOLink';

function DeleteGPOLink;     external GPEditLib name 'DeleteGPOLink';
function DeleteAllGPOLinks; external GPEditLib name 'DeleteAllGPOLinks';

function BrowseForGPO;      external GPEditLib name 'BrowseForGPO';

function ImportRSoPData;    external GPEditLib name 'ImportRSoPData';
function ExportRSoPData;    external GPEditLib name 'ExportRSoPData';


end.

